<?php



/**
 * This behavior extends the event dispatching mechanism of the attached
 * component. In addition to invoke local callbacks, they may now also signal
 * remote services.
 *
 * @author Benjamin
 */
class SignalBehavior extends CBehavior
{
  public $events = array('*');

  /**
   * @var EventInterceptor
   */
  private $_eventInterceptor = null;

  public function attach($owner)
  {
    parent::attach( $owner );

    $this->_eventInterceptor = new EventInterceptor();
    $this->_eventInterceptor->initialize( $owner, $this->events );
    $this->_eventInterceptor->onEventIntercepted = array( $this, 'emitSignals' );
  }

  /**
   * We will reach this method, whenever $owner raises an event. So, we'll
   * lookup registered observers, and signal them.
   *
   * @param InterceptionEvent $event
   */
  public function emitSignals( InterceptionEvent $event )
  {
    $signalId = $this->getSignalId( $event );

    /* @var $emitter SignalEmitterI */
    foreach ($this->getEmitters($signalId) as $emitter)
      $emitter->emit( $event->getInterceptedEvent() );
  }

  /**
   * Returns the signal id. It is defined as the classname (with namespace if
   * existent), followed by '\', followed by the event name.
   *
   * So the signal "Shop\User\onRegistered" would be emitted, whenever the
   * event "onRegistered" of the class "User" in namespace "Shop" would be
   * raised.
   *
   * @param InterceptionEvent $event
   * @return string
   */
  protected function getSignalId( InterceptionEvent $event )
  {
    $senderClassName      = get_class($event->getInterceptedEvent()->sender);
    $intercepedEventName  = $event->getInterceptedEventName();
    return $senderClassName.'\\'.$intercepedEventName;
  }

  /**
   * @param string $signalId
   * @return SignalObserverI[]
   */
  protected function getEmitters( $signalId )
  {
    $emitters = array();
    $arSignalEmitters = ARSignalEmitters::model()->findAll(
      'signalId = :signalId', array(
        ':signalId' => $signalId,
      )
    );

    /* @var $arSignalEmitter ARSignalEmitter */
    foreach ( $arSignalEmitters as $arSignalEmitter )
    {
      $emitter = EmitterFactory::createEmitter( $arSignalEmitter );

      if ($emitter instanceof SignalEmitterI)
        $emitters[] = $emitter;
    }

    return $emitters;
  }

}
