<?php

/**
 * Description of ARSignalEmitters
 *
 * @author Benjamin
 */
class ARRestSignalEmitters extends ARSignalEmitters
{
  private $_connectionKeyPath = '';
  private $_connectionKeyModel = null;

	/**
	 * Returns the static model of the specified AR class.
	 * @return ARSignalEmitters the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

  protected function afterConstruct()
  {
    parent::afterConstruct();

    $this->type = EmitterType::REST;
    $this->setEmitterConfig( new RestEmitterConfigDTO() );
  }

  protected function afterSave()
  {
    parent::afterSave();

    if ($this->_connectionKeyModel instanceof ARConnectionKeys)
    {
      $this->_connectionKeyModel->delete();
      $this->_connectionKeyModel = null;
    }
  }

  protected function afterDelete()
  {
    parent::afterDelete();

    if ($this->_connectionKeyModel instanceof ARConnectionKeys)
    {
      $this->_connectionKeyModel->delete();
      $this->_connectionKeyModel = null;
    }
  }

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array_merge(parent::rules(), array(
      array('hostname', 'validateConnectionKey', 'on' => 'RestConnect, RestDisconnect' ),
		));
	}

  public function validateConnectionKey( $attribute, $params )
  {
    $config = $this->getEmitterConfig();
    if (!$config instanceof RestEmitterConfigDTO) {
      $this->addError( 'hostname', "can't validate the connection key." );
      return;
    }

    $aUrlParts = parse_url( $config->url );
    $hostname = array_key_exists( 'host', $aUrlParts ) ? $aUrlParts['host'] : '';
    $this->_connectionKeyModel = ARConnectionKeys::model()->find('hostname=:hostname',array(':hostname'=>$hostname));
    if (!$this->_connectionKeyModel instanceof ARConnectionKeys) {
      $this->addError( 'hostname', "can't validate the connection key." );
      return;
    }

    $scheme = array_key_exists( 'scheme', $aUrlParts ) ? $aUrlParts['scheme'] : 'http';
    $port = '80';
    if (array_key_exists('port',$aUrlParts))
      $port = $aUrlParts['port'];
    else if ($scheme === 'https')
      $port = '443';
    $resource = "{$scheme}://{$hostname}:{$port}/";
    if ($this->_connectionKeyPath !== '')
      $resource .= "{$this->_connectionKeyPath}/";
    $resource .= $this->_connectionKeyModel->key;

    $headers = get_headers( $resource );

    $ok = is_array($headers);
    if ($ok)
      $ok = array_key_exists(0,$headers);
    if ($ok)
      $ok = $headers[0] === 'HTTP/1.1 200 OK' || $headers[0] === 'HTTP/1.0 200 OK';

    if (!$ok)
      $this->addError( 'hostname', "can't validate the connection key." );
  }

  /**
   * @param string url
   */
  public function setUrl( $url )
  {
    $config = $this->getEmitterConfig();
    $config->url = $url;
    $this->setEmitterConfig( $config );

    $aUrlParts = parse_url( $url );
    if (!is_array($aUrlParts))
    {
      $this->hostname = '';
      $this->filepath = '';
    }
    else
    {
      $this->hostname = $this->getHostname( $url );
      $this->filepath = array_key_exists('path', $aUrlParts) ? $aUrlParts['path'] : '';
    }
  }

  /**
   * @param string $connectionKeyPath
   */
  public function setConnectionKeyPath( $connectionKeyPath )
  {
    $this->_connectionKeyPath = $connectionKeyPath;
  }

  static public function getHostname( $url )
  {
    $aUrlParts = @parse_url( $url );

    if (!is_array($aUrlParts))
      return '';
    else
    {
      $hostname = '';
      $scheme = $aUrlParts['scheme'];
      switch ($scheme)
      {
      case 'file':
          $hostname = 'localhost';
          break;
      default:
          $hostname = array_key_exists('host',$aUrlParts) ? $aUrlParts['host'] : '';
          break;
      }

      return $hostname;
    }
  }

}
